/*
** Generated by X-Designer, then edited heavily by Bob Hays, SPSS Inc. 1993
**
** Welcome to motifbox, a little tool for script programmers.
**
** This tool takes up to four command arguments and displays an
** OSF/Motif message box.  The box can have any of the allowed
** types (use the -type flag):
**
** FLAG        DIALOG TYPE
** ====        ===========
** error       XmDIALOG_ERROR
** message     XmDIALOG_MESSAGE
** info        XmDIALOG_INFORMATION
** warning     XmDIALOG_WARNING
** question    XmDIALOG_QUESTION
** working     XmDIALOG_WORKING
**
** For all the boxes accept question, one push button (OK) is left
** displayed for the user to put the box away.
**
** Question boxes leave up two push buttons, yes and no.  The default
** button is selected with the -def switch, the only allowed value
** is no, indicating use the no button instead of the yes button.
**
** The -message switch determines the text for the label area.
**
** If the constant WRAP is defined at compile time, then the -fold
** switch value, an integer, is used to fold, or wrap, text at or
** near that byte location.  Otherwise the message is displayed in one line.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** A copy of the GNU General Public License can be obtained from this
** program's author (send electronic mail to kyle@cs.odu.edu) or from
** the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
** 02139, USA.
*/
/*
**LIBS: -lXm -lXt -lX11
*/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/Protocols.h>
#include <Xm/DialogS.h>
#include <Xm/MessageB.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <stdio.h>

#define CLASSNAME "MotifBox"
#define YES       "Yes"
#define NO        "No"
#define NO_LOWER  "no"

#include "motifbox-resources.h"

Widget       widget0      = (Widget) NULL;                                      /* The shell widget. */
Widget       MessageBox   = (Widget) NULL;                                      /* The MessageBox container. */
Widget       HelpButton   = (Widget) NULL;                                      /* The Help button. */
Widget       MessageLabel = (Widget) NULL;                                      /* The Message area. */
Widget       CancelButton = (Widget) NULL;                                      /* The Cancel (No) button. */
Widget       OkayButton   = (Widget) NULL;                                      /* The Okay (Yes) button. */
XtAppContext app_context;                                                       /* Application context. */
Display      *display;                                                          /* Display. */

/*******************************************************************/
/* strlwr                                                          */
/*                                                                 */
/* Lowercase a null-terminated string.                             */
/*******************************************************************/
char *strlwr ( String string )
{
    char *tmp;

    for ( tmp = string; *tmp != '\0'; tmp++ )                                   /* Single byte only here. */
    {
        *tmp = (char) tolower ( (int) *tmp );
    }
}

/*******************************************************************/
/* DeleteButton                                                    */
/*                                                                 */
/* Destroys the widget for a push button.                          */
/*******************************************************************/
void DeleteButton ( Widget widget )
{
    XtDestroyWidget ( widget );
    return;
}

/*******************************************************************/
/* NoCallback                                                      */
/*                                                                 */
/* Motif callback function for CancelButton.                       */
/*******************************************************************/
void NoCallback ()
{
    exit ( 2 );
}

/*******************************************************************/
/* OkayCallback                                                    */
/*                                                                 */
/* Motif callback function for OkayButton.                         */
/*******************************************************************/
void OkayCallback ()
{
    exit ( 0 );
}

/*******************************************************************/
/* SetMessageBoxLabel                                              */
/*                                                                 */
/* Changes the label message of the message box to the one entered */
/* on the command line.                                            */
/*******************************************************************/
void SetMessageBoxLabel ( Widget labelWidget, String message, int fold_location )
{
    Arg               al[2];
    Cardinal          ac = 0;
    XmString          messageValue = (XmString) NULL;                           /* Start with nothin'. */
#ifdef WRAP
    XmString          lastString;                                               /* Temporary store for messageValue. */
    XmString          newString;                                                /* Temporary store for other stuff. */
    register int      startLoc = 0;
    register int      i;
    char              *workSpace = (char *) NULL;
#endif

#ifndef WRAP
    /*******************************************************************/
    /* This block of code provides the simplest                        */
    /* form of message - just put it out there.                        */
    /*******************************************************************/
    if ( ( messageValue = XmStringCreate ( message,                             /* Use message string. */
                                           XmSTRING_DEFAULT_CHARSET ) ) != (XmString) NULL )
    {
        XtSetArg ( al[ac], XmNlabelString, messageValue ); ac++;                /* Move label into arg list now. */
        XtSetValues ( labelWidget, al, ac );                                    /* Send string to widget for use. */
        XmStringFree ( messageValue );                                          /* Put the string away now. */
    }
#else /* Wrap lines then. */
    /***********************************************************************/
    /* Here we will wrap lines.  Note the annoying XmString constructions. */
    /***********************************************************************/
    if ( ( workSpace = malloc ( strlen ( message ) + 1 ) ) != (char *) NULL )
    {
        strcpy ( workSpace, message );                                          /* Copy message to working storage now. */
        if ( strlen ( workSpace ) > fold_location )                             /* The string will require folding. */
        {
            while ( workSpace[0] != '\0' &&
                    strlen ( workSpace ) > fold_location )
            {
                for ( i = fold_location; i > 0; i-- )                           /* Look for blank to break on. */
                {
                    if ( workSpace[i] == ' ' )
                    {
                        workSpace[i] = '\0';
                        if ( messageValue == (XmString) NULL )                  /* First time through loop. */
                        {
                            messageValue = XmStringCreate ( workSpace, XmSTRING_DEFAULT_CHARSET );
                        }
                        else
                        {
                            newString = XmStringSeparatorCreate ( );
                            lastString = XmStringCopy ( messageValue );
                            XmStringFree ( messageValue );
                            messageValue = XmStringConcat ( lastString, newString );
                            XmStringFree ( lastString );
                            XmStringFree ( newString );

                            newString = XmStringCreate ( workSpace, XmSTRING_DEFAULT_CHARSET );
                            lastString = XmStringCopy ( messageValue );
                            XmStringFree ( messageValue );
                            messageValue = XmStringConcat ( lastString, newString );
                            XmStringFree ( lastString );
                            XmStringFree ( newString );
                        }
                        workSpace += i + 1;
                        break;
                    }
                }
                if ( i == startLoc )
                {
                    workSpace += fold_location;                                 /* Move over this much. */
                }
            }
        }

        /*******************************************************************/
        /* Make sure we do the last segment too.                           */
        /*******************************************************************/
        if ( messageValue == (XmString) NULL )
        {
            messageValue = XmStringCreate ( workSpace, XmSTRING_DEFAULT_CHARSET );
        }
        else
        {
            newString = XmStringSeparatorCreate ( );
            lastString = XmStringCopy ( messageValue );
            XmStringFree ( messageValue );
            messageValue = XmStringConcat ( lastString, newString );
            XmStringFree ( lastString );
            XmStringFree ( newString );
            
            newString = XmStringCreate ( workSpace, XmSTRING_DEFAULT_CHARSET );
            lastString = XmStringCopy ( messageValue );
            XmStringFree ( messageValue );
            messageValue = XmStringConcat ( lastString, newString );
            XmStringFree ( lastString );
            XmStringFree ( newString );
        }

        /*******************************************************************/
        /* Okay, now we need to load the widget with the string.           */
        /*******************************************************************/
        if ( messageValue != (XmString) NULL )                                  /* Yep, we got a message in here. */
        {
            XtSetArg ( al[ac], XmNlabelString, messageValue ); ac++;            /* Move label into arg list now. */
            XtSetValues ( labelWidget, al, ac );                                /* Send string to widget for use. */
            XmStringFree ( messageValue );                                      /* Put the string away now. */
        }
        free ( workSpace );
    }
#endif
}

/*******************************************************************/
/* SetMessageBoxType                                               */
/*                                                                 */
/* Changes the icon and buttons to match the type of box the user  */
/* asked for.                                                      */
/*******************************************************************/
void SetMessageBoxType ( Display *display, Widget messageBox, Widget okayButton,
                         Widget cancelButton, String type,
                         String defButton )
{
    Arg           al[5];
    Cardinal      ac = 0;
    struct
    {
        String          value;
        unsigned char   type;
    } table[] =
    {
    { "error",       XmDIALOG_ERROR },
    { "info",        XmDIALOG_INFORMATION },
    { "message",     XmDIALOG_MESSAGE },
    { "question",    XmDIALOG_QUESTION },
    { "warning",     XmDIALOG_WARNING },
    { "working",     XmDIALOG_WORKING },
    { (String) NULL, (unsigned char) 0 }
    };
    int           counter = 0;
    char          *typeCopy = (char *) NULL;
    char          *defButtonCopy = (char *) NULL;
    XmString      buttonLabel1 = (XmString) NULL;
    Boolean       done = (Boolean) FALSE;
    Atom          WM_DELETE_WINDOW;                                                 /* Atom for WM deletion. */
    unsigned char defaultButton;

    /*******************************************************************/
    /* Executable begins here.                                         */
    /*                                                                 */
    /* First, we copy the type switch value that the user              */
    /* entered and lower case it.  If there is no value for            */
    /* the switch, then we use the first type code in the              */
    /* table.                                                          */
    /*******************************************************************/
    if ( type != (char *) NULL )                                                /* We got a type switch on the command line. */
    {
        if ( ( typeCopy = malloc ( strlen ( type ) + 1 ) ) != (char *) NULL )
        {
            strcpy ( typeCopy, type );                                          /* Copy type to work space. */
            (void) strlwr ( typeCopy );                                         /* Lower-case work space. */
        }
    }
    else
    {
        if ( ( typeCopy = malloc ( strlen ( table[0].value ) + 1 ) ) != (char *) NULL )
        {
            strcpy ( typeCopy, table[0].value );
        }
    }
            
    /*******************************************************************/
    /* Now, we compare the user-entered type to the table              */
    /* values; if we find a match, we check if it is the               */
    /* question box - this requires loading the yes and no             */
    /* labels into the okay and cancel buttons and set the             */
    /* proper button as the default.  Otherwise,                       */
    /* we delete the cancel button.                                    */
    /*******************************************************************/
    while ( !done && typeCopy != (char *) NULL )                                /* This takes care of invalid type values. */
    {
        for ( counter = 0; table[counter].value != (char *) NULL; counter++ )
        {
            if ( strcmp ( typeCopy, table[counter].value ) == 0 )               /* Got a matching value. */
            {
                if ( table[counter].type != XmDIALOG_QUESTION )
                {
                    DeleteButton ( cancelButton );
                }
                else
                {
                    /*******************************************************************/
                    /* First we will load the right labels for the buttons.            */
                    /*******************************************************************/
                    if ( ( buttonLabel1 = XmStringCreate ( YES,
                                                           XmSTRING_DEFAULT_CHARSET ) ) != (XmString) NULL )
                    {
                        ac = 0;
                        XtSetArg ( al[ac], XmNlabelString, buttonLabel1 ); ac++;
                        XtSetValues ( okayButton, al, ac );
                        XmStringFree ( buttonLabel1 );
                    }
                    if ( ( buttonLabel1 = XmStringCreate ( NO,
                                                           XmSTRING_DEFAULT_CHARSET ) ) != (XmString) NULL )
                    {
                        ac = 0;
                        XtSetArg ( al[ac], XmNlabelString, buttonLabel1 ); ac++;
                        XtSetValues ( cancelButton, al, ac );
                        XmStringFree ( buttonLabel1 );
                    }

                    /*******************************************************************/
                    /* Now, we set the right default, if we can do that.               */
                    /*******************************************************************/
                    if ( defButton != (char *) NULL &&
                         defButton[0] != '\0' &&
                         ( defButtonCopy = malloc ( strlen ( defButton ) + 1 ) ) != (char *) NULL )
                    {
                        strcpy ( defButtonCopy, defButton );                    /* Copy value to temporary buffer. */
                        strlwr ( defButtonCopy );                               /* Lowercase to compare apples and apples. */
                        if ( strcmp ( defButtonCopy, NO_LOWER ) == 0 )          /* Matched no, so change it. */
                        {
                            ac = 0;
                            XtSetArg ( al[ac], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON ); ac++;
                            XtSetValues ( messageBox, al, ac );
                        }
                        free ( defButtonCopy );                                 /* Give back memory. */
                    }

                    /*******************************************************************/
                    /* And, finally, we set up the window manager handler for          */
                    /* deleting the window from the main menu - tie it to              */
                    /* the default button.                                             */
                    /*******************************************************************/
                    WM_DELETE_WINDOW = XmInternAtom ( display,
                                                      "WM_DELETE_WINDOW",
                                                      (Boolean) True );
                    ac = 0;
                    XtSetArg ( al[ac], XmNdefaultButtonType, &defaultButton ); ac++;
                    XtGetValues ( messageBox, al, ac );                         /* Get the default button type for dialog. */
                    if ( defaultButton == XmDIALOG_CANCEL_BUTTON )
                    {
                        XmAddWMProtocolCallback ( widget0, WM_DELETE_WINDOW,
                                                  NoCallback, (caddr_t) NULL );
                    }
                    else
                    {
                        XmAddWMProtocolCallback ( widget0, WM_DELETE_WINDOW,
                                                  OkayCallback, (caddr_t) NULL );
                    }
                }
                ac = 0;
                XtSetArg ( al[ac], XmNdialogType, table[counter].type ); ac++;
                XtSetValues ( messageBox, al, ac );
                done = TRUE;                                                    /* Indicate we found match, stop now. */
                break;                                                          /* Stop the loop now. */
            }
        }

        /*******************************************************************/
        /* We only get here if the user entered an invalid                 */
        /* type value.  We'll use the first table entry                    */
        /* in that case.                                                   */
        /*******************************************************************/
        if ( !done )                                                            /* Failed to find a match. */
        {
            free ( typeCopy );                                                  /* Give memory back for invalid value. */
            if ( ( typeCopy = malloc ( strlen ( table[0].value ) + 1 ) ) != (char *) NULL )
            {
                strcpy ( typeCopy, table[0].value );
            }
        }
    }
    if ( typeCopy != (char *) NULL )
    {
        free ( typeCopy );
    }
}

/*******************************************************************/
/* CreateWidgets                                                   */
/*                                                                 */
/* Makes the widgets we need.                                      */
/*******************************************************************/
void CreateWidgets ( Display *display, char *app_name )
{
    Widget children[6];                                                         /* Children to manage */
    Arg al[64];                                                                 /* Arg List */
    register int ac = 0;                                                        /* Arg Count */
    char from_s [256];                                                          /* For font list conversion */
    XrmValue from_value, to_value;                                              /* ditto */
    int fg, bg;                                                                 /* colour values for pixmaps */ 
    XmString *list_items;                                                       /* For list items */
    int list_item;                                                              /* Index for list_items */
    XmString xmstrings[15];                                                     /* temporary storage for XmStrings */

    /*******************************************************************/
    /* Create the widget we need - XmMessageBox.                       */
    /*******************************************************************/
    ac = 0;
    XtSetArg ( al[ac], XmNdialogType, XmDIALOG_ERROR ); ac++;                   /* Default type of message box - error. */
    XtSetArg ( al[ac], XmNmessageAlignment, XmALIGNMENT_CENTER ); ac++;         /* Center each line of the label. */
    MessageBox = XmCreateMessageBox ( widget0, "MessageBox", al, ac );          /* Create the message box. */
    ac = 0;

    /*******************************************************************/
    /* Now retrieve the children and destroy the help button.          */
    /*******************************************************************/
    CancelButton = XmMessageBoxGetChild ( MessageBox, XmDIALOG_CANCEL_BUTTON );
    HelpButton = XmMessageBoxGetChild ( MessageBox, XmDIALOG_HELP_BUTTON );
    MessageLabel = XmMessageBoxGetChild ( MessageBox, XmDIALOG_MESSAGE_LABEL );
    OkayButton = XmMessageBoxGetChild ( MessageBox, XmDIALOG_OK_BUTTON );
    XtAddCallback ( MessageBox, XmNcancelCallback, NoCallback, NULL );
    XtAddCallback ( MessageBox, XmNokCallback, OkayCallback, NULL );
    XtAddCallback ( CancelButton, XmNactivateCallback, NoCallback, NULL );
    DeleteButton ( HelpButton );                                                /* Remove this button always - no help here. */
    XtAddCallback ( OkayButton, XmNactivateCallback, OkayCallback, NULL );
    XtManageChild ( MessageBox);
}


int main ( unsigned int argc, char **argv)
{
    Arg               al[5];                                                    /* Arg List */
    register Cardinal ac = 0;                                                   /* Arg Count */

    /*******************************************************************/
    /* The normal first stuff to do.                                   */
    /*******************************************************************/
    XtSetArg(al[ac], XmNallowShellResize, TRUE); ac++;
    if ( ( widget0 = XtAppInitialize ( &app_context,
                                       CLASSNAME,
                                       (XrmOptionDescRec *) &OptionList,
                                       (Cardinal) OptionCount,
                                       (Cardinal *) &argc,
                                       (String *) argv,
                                       (String *) NULL,
                                       al,
                                       ac ) ) == (Widget) NULL )
    {
        printf("%s: OSF/Motif initialization failure, exiting...\n", argv[0]);
        exit (-1);
    }
    display = XtDisplay ( widget0 );
    XtGetApplicationResources ( widget0,                                        /* Load up special resources now. */
                                &ResourceValues,
                                Resources,
                                ResourceCount,
                                (ArgList *) NULL,
                                (Cardinal) 0 );

    /*******************************************************************/
    /* Register converters, just in case you are really unlucky !!     */
    /*******************************************************************/
    XmRegisterConverters();
    XtAddConverter ( XmRString,                                                 /* String to unit type doesn't get added !! */
                     XmRUnitType,
                     XmCvtStringToUnitType,
                     NULL,
                     0 );

    /*******************************************************************/
    /* Create our widgets now.                                         */
    /*******************************************************************/
    CreateWidgets ( display, argv[0] );

    /*******************************************************************/
    /* Analyze the resource settings now.                              */
    /*******************************************************************/
    if ( ResourceValues.message != (String) NULL &&
         MessageLabel != (Widget) NULL )
    {
        SetMessageBoxLabel ( MessageLabel,                                      /* Set up label area. */
                             ResourceValues.message,
                             ResourceValues.fold );
    }
    if ( ResourceValues.type != (String) NULL &&
         MessageBox != (Widget) NULL &&
         CancelButton != (Widget) NULL &&
         OkayButton != (Widget) NULL )
    {
        SetMessageBoxType ( display,
                            MessageBox,                                         /* Set up type of message box. */
                            OkayButton,
                            CancelButton,
                            ResourceValues.type,
                            ResourceValues.defButton );
    }

    /*******************************************************************/
    /* Display the box and loop forever.                               */
    /*******************************************************************/
    XtRealizeWidget (widget0);
    XtAppMainLoop (app_context);
    exit (0);
}

